## Nondeterministic MuZero (NDMZ)

### Areas of Interest
* Look in `nannon/params.py` to see or change all training parameters.
* Look in `algorithms/mu_zero/bot.py` to see the tree search.
* Look in `algorithms/mu_zero/worker.py` to see the the ray worker logic.
* Look in `algorithms/services/train` and `algorithms/mu_zero/evaluator.py` to see the training loop.
* Look in `algorithms/services/play` and `algorithms/mu_zero/gymnasium.py` to see the self-play.
* Look in `algorithms/services/eval` and `algorithms/mu_zero/lab.py` to see the dynamics test.
* Look in `algorithms/bots/bellman.py` to see the optimal agent.
* Look in `nannon/extractor.py` to see the action-to-image logic.
* Look in `minispiel/games/nannon.h` for Nannon C++ code. 

### Instructions for replication on a cluster
* Install `ray` version 0.8.7 in a virtual environment.
* Start a demo CloudAMQP account. (Or sub in your own service that works with `pika`).
* Start up a Kubernetes cluster with at least one GPU. The default setup uses one GPU node and ten 96 vCPU nodes.
* In `Latest.Dockerfile`, fill in `CUDA_VISIBLE_DEVICES` and your AMQP connection string
* Build `Base.Dockerfile` and `Latest.Dockerfile`
* Push to DockerHub
* Paste your dockerhub path in lines 159 and 237 of `ray/eval.yaml`, `ray/play.yaml`, and `ray/train.yaml`

The algorithm is split into three services for self-play, training, and evaluation.
The settings in the `.yaml` files are what we used.
But you can change the number of workers and CPU/GPU requirements as you wish. 

* Run `ray up ray/train.yaml`
* Run `ray attach ray/train.yaml`
* Run `./scripts/mu_6_3_6/train.sh`
(You can change the start commands on line 307 of the `.yaml` files if you don't want to attach)

After exporting the AMQP string to your local computer's environment variables (or some machine with persistent store),
run `./scripts/mu_6_3_6/backup.sh`. This service will receive the weights and logs.

* In a separate terminal, run `ray up ray/play.yaml`
* Run `ray attach ray/play.yaml`
* Run `./scripts/mu_6_3_6/play.sh`

* In a separate terminal, run `ray up ray/eval.yaml`
* Run `ray attach ray/eval.yaml`
* Run `./scripts/mu_6_3_6/eval.sh`

The services should now be running and commincating via AMQP.
You should receive logs and weights via the backup service locally.

You can also run the program locally. It requires building a pyspiel object first via
`./minispiel/scripts/build_and_run_tests.sh` and adding the `build/python/` directory that
is created to your PYTHONPATH. See the OpenSpiel GitHub for more details.
