from __future__ import annotations
from abc import ABC, abstractmethod
from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from algorithms.utils.types import SpielAction, ActionImage, SpielState, TauPolicy, ChancePolicy, ChoicePolicy, \
    SpielGame
    from algorithms.utils.types import TrainingTarget


class Extractor(ABC):
    def __init__(self, game: SpielGame, num_actions: int, pass_action: SpielAction, num_players: int):
        self._game = game
        self._num_distinct_actions = game.num_distinct_actions()
        self._num_actions = num_actions
        self._pass_action = pass_action
        self._num_players = num_players

    @abstractmethod
    def action_to_image(self, action: SpielAction) -> ActionImage:
        raise NotImplementedError

    @abstractmethod
    def state_feature_extractor(self, state: SpielState) -> ActionImage:
        raise NotImplementedError

    @abstractmethod
    def final_action_image(self) -> ActionImage:
        raise NotImplementedError

    @abstractmethod
    def final_target(self) -> TrainingTarget:
        raise NotImplementedError

    @abstractmethod
    def tau_policy_deterministic(self, player_id: int) -> TauPolicy:
        raise NotImplementedError

    @abstractmethod
    def chance_policy(self, probs: List[float]) -> ChancePolicy:
        raise NotImplementedError

    @abstractmethod
    def chance_policy_deterministic(self, spiel_action: SpielAction) -> ChancePolicy:
        raise NotImplementedError

    @abstractmethod
    def choice_policy(self, probs: List[float]) -> ChoicePolicy:
        raise NotImplementedError

    @abstractmethod
    def choice_policy_deterministic(self, spiel_action: SpielAction) -> ChoicePolicy:
        raise NotImplementedError

