from abc import ABC, abstractmethod
from typing import Sized


class GameHistory(ABC, Sized):
    """
    The GameHistory object stores the results of self-training produced by the self-play service. When sent to the
    training service, it is stored in a ReplayBuffer
    """

    @abstractmethod
    def __len__(self) -> int:
        """
        Must provide length method
        Returns:
            an integer for the number of history items
        """
        raise NotImplementedError

    @abstractmethod
    def store(self, *args, **kwargs) -> None:
        """
        Must be defined for storing game history items
        """
        raise NotImplementedError

