from __future__ import annotations
from abc import ABC, abstractmethod
from algorithms.abstract import ZeroBot
from algorithms.abstract.game_history import GameHistory
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from algorithms.utils.types import SpielGame
    from algorithms.utils.params import Params


class Gymnasium(ABC):
    def __init__(self, game: SpielGame, bellman_bot: ZeroBot, params: Params):
        """
        Used for performing self-play games with the given bot.
        Args:
            game: `SpielGame`, a pyspiel game object
            params: `GameParams`, a named tuple with game parameter information
            bot: `ZeroBot`, a bot agent that makes decisions at states.
        """
        self._game = game
        self._bellman_bot = bellman_bot
        self._helper = params.extractor
        self.steps = 0

    @abstractmethod
    def self_play_single(self, bot: ZeroBot, with_nodes=False) -> GameHistory:
        """
        Performs one game of self-play, recording training targets and other information about the game.
        Returns:
            A `GameHistory` object with information about the game.
        """
        raise NotImplementedError

