from typing import List

from algorithms.abstract.game_history import GameHistory
from algorithms.utils.types import AlphaZeroResult


class AlphaZeroGameHistory(GameHistory):
    def __init__(self) -> None:
        self._results = []

    def __len__(self) -> int:
        return len(self._results)

    def store(self, result: AlphaZeroResult) -> None:
        """
        Stores the training target results taken from a game of self-play
        Args:
            result: `AlphaZeroResult`, a named tuple with state feature, policy target, and value target
        """
        self._results.append(result)

    def get_results(self) -> List[AlphaZeroResult]:
        """
        Extracts the results for use by the ReplayBuffer.
        Returns:
            a list of AlphaZeroResult items
        """
        return self._results


