from algorithms.abstract import ZeroWorker
from algorithms.alpha_zero import AlphaZeroBot, AlphaZeroEvaluator, AlphaZeroGymnasium
import ray

from algorithms.bots import BellmanBot, RandomBot
from algorithms.utils.types import SpielGame, DynamicsTestLog
from algorithms.utils.params import Params


@ray.remote
class AlphaZeroWorker(ZeroWorker):
    def __init__(self, i: int, game: SpielGame, params: Params):
        evaluator = AlphaZeroEvaluator(params)
        zero_bot = AlphaZeroBot(game, evaluator, params)
        bellman_bot = BellmanBot(game, params)
        random_bot = RandomBot(game, params)
        gymnasium = AlphaZeroGymnasium(game, bellman_bot, params)
        ZeroWorker.__init__(self, i, game, evaluator, (zero_bot, bellman_bot, random_bot), gymnasium, params)

    def set_replay_buffer(self, replay_buffer):
        self._replay_buffer = replay_buffer

    def eval_dynamics(self, _):
        dummy_log = DynamicsTestLog(choice_top_acc=[1], choice_pass_action=[1], chance_pass_action=[1],
                                    choice_single_acc=[1], choice_strict_acc=[1], tau_acc=[1])
        return dummy_log


