import numpy as np
from typing import Any, Tuple
from algorithms.abstract.bot import ZeroBot
from algorithms.utils.params import Params
from algorithms.utils.types import SpielState, SpielGame, SpielAction, ChoicePolicy


class RandomBot(ZeroBot):
    """A random bot."""

    def __init__(self, game: SpielGame, params: Params, verbose: bool = False):
        ZeroBot.__init__(self, game, params, verbose)

    def step(self, state: SpielState) -> SpielAction:
        return np.random.choice(state.legal_actions())

    def action_and_policy(self, state: SpielState) -> Tuple[SpielAction, ChoicePolicy]:
        legal_actions = state.legal_actions()
        action = np.random.choice(legal_actions)
        probs = []
        for a in range(state.num_distinct_actions()):
            prob = 1.0 / len(legal_actions) if a in legal_actions else 0.0
            probs.append(prob)
        choice_policy_target = self._extractor.choice_policy(probs)
        return action, choice_policy_target

    def search(self, state: SpielState) -> Any:
        pass
